@@ -10,15 +10,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk, GLib, Pango
 
 from lollypop.logger import Logger
 from lollypop.selectionlist import SelectionList
-from lollypop.define import App, Type, SelectionListMask
+from lollypop.define import App, Type, SelectionListMask, StorageType, ViewType
 from lollypop.shown import ShownLists
 from lollypop.helper_gestures import GesturesHelper
 from lollypop.view import View
-from lollypop.utils import emit_signal
+from lollypop.utils import emit_signal, get_default_storage_type, get_icon_name
 
 
 class NoneView(View):
@@ -26,6 +26,9 @@ class NoneView(View):
         A view that do nothing
     """
 
+    def __init__(self):
+        View.__init__(self, StorageType.COLLECTION, ViewType.DEFAULT)
+
     @property
     def args(self):
         return None
@@ -147,10 +150,8 @@ class ListsContainer:
             @param genre_ids as [int]
         """
         def load():
-            if App().settings.get_value("show-performers"):
-                artists = App().artists.get_performers(genre_ids)
-            else:
-                artists = App().artists.get(genre_ids)
+            storage_type = get_default_storage_type()
+            artists = App().artists.get(genre_ids, storage_type)
             return artists
         selection_list.set_mask(SelectionListMask.ARTISTS)
         App().task_helper.run(load, callback=(selection_list.populate,))
@@ -212,6 +213,7 @@ class ListsContainer:
             self.left_list.hide()
             self.left_list.clear()
 
+        storage_type = get_default_storage_type()
         if selected_id in [Type.ARTISTS_LIST, Type.GENRES_LIST] and not\
                 App().window.is_adaptive:
             view = NoneView()
@@ -225,26 +227,54 @@ class ListsContainer:
         elif selected_id == Type.SEARCH:
             view = self.get_view_search()
         elif selected_id == Type.SUGGESTIONS:
-            view = self._get_view_suggestions()
+            view = self._get_view_suggestions(storage_type)
         elif selected_id in [Type.POPULARS,
                              Type.LOVED,
                              Type.RECENTS,
                              Type.LITTLE,
-                             Type.RANDOMS,
-                             Type.WEB]:
-            view = self._get_view_albums([selected_id], [])
+                             Type.RANDOMS]:
+            view = self._get_view_albums([selected_id], [], storage_type)
+        elif selected_id == Type.WEB:
+            view = self._get_view_albums([selected_id], [], StorageType.SAVED)
         elif selected_id == Type.RADIOS:
-            view = self._get_view_radios()
+            message = """Radio support has been removed, sorry for that.
+            For a better radio player, use Shortwave:
+            """
+            view = View(StorageType.ALL, ViewType.DEFAULT)
+            image = Gtk.Image.new_from_icon_name(get_icon_name(Type.RADIOS),
+                                                 Gtk.IconSize.INVALID)
+            image.set_pixel_size(256)
+            image.set_property("expand", True)
+            style = image.get_style_context()
+            style.add_class("dim-label")
+            label = Gtk.Label()
+            style = label.get_style_context()
+            style.add_class("dim-label")
+            style.add_class("text-xx-large")
+            label.set_markup("%s" % GLib.markup_escape_text(message))
+            label.set_line_wrap_mode(Pango.WrapMode.WORD)
+            label.set_line_wrap(True)
+            button = Gtk.LinkButton.new(
+                "https://flathub.org/apps/details/de.haeckerfelix.Shortwave")
+            grid = Gtk.Grid()
+            grid.set_valign(Gtk.Align.CENTER)
+            grid.set_row_spacing(20)
+            grid.set_orientation(Gtk.Orientation.VERTICAL)
+            grid.add(image)
+            grid.add(label)
+            grid.add(button)
+            view.add_widget(grid)
+            view.show_all()
         elif selected_id == Type.YEARS:
-            view = self._get_view_albums_decades()
+            view = self._get_view_albums_decades(storage_type)
         elif selected_id == Type.GENRES:
-            view = self._get_view_genres()
+            view = self._get_view_genres(storage_type)
         elif selected_id == Type.ARTISTS:
-            view = self._get_view_artists_rounded()
+            view = self._get_view_artists_rounded(storage_type)
         elif selected_id == Type.ALL:
-            view = self._get_view_albums([selected_id], [])
+            view = self._get_view_albums([selected_id], [], storage_type)
         elif selected_id == Type.COMPILATIONS:
-            view = self._get_view_albums([selected_id], [])
+            view = self._get_view_albums([selected_id], [], storage_type)
         if view is not None and view not in self._stack.get_children():
             view.show()
             self._stack.add(view)
@@ -281,13 +311,14 @@ class ListsContainer:
         Logger.debug("Container::__on_left_list_activated()")
         selected_ids = self.left_list.selected_ids
         view = None
+        storage_type = get_default_storage_type()
         if self.left_list.mask & SelectionListMask.GENRES:
             if not App().window.is_adaptive:
-                view = self._get_view_albums(selected_ids, [])
+                view = self._get_view_albums(selected_ids, [], storage_type)
             self._show_artists_list(self.right_list, selected_ids)
             self._show_right_list()
         else:
-            view = self._get_view_artists([], selected_ids)
+            view = self._get_view_artists([], selected_ids, storage_type)
             self.set_focused_view(view)
         if view is not None:
             view.show()
@@ -300,9 +331,10 @@ class ListsContainer:
             @param listbox as Gtk.ListBox
             @param row as Gtk.ListBoxRow
         """
+        storage_type = get_default_storage_type()
         genre_ids = self.left_list.selected_ids
         artist_ids = self.right_list.selected_ids
-        view = self._get_view_artists(genre_ids, artist_ids)
+        view = self._get_view_artists(genre_ids, artist_ids, storage_type)
         view.show()
         self._stack.add(view)
         self._stack.set_visible_child(view)
