@@ -10,10 +10,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib
+from gi.repository import GLib, Gtk
 
-from lollypop.define import ViewType
+from lollypop.define import ViewType, App
 from lollypop.utils import noaccents
+from lollypop.utils_album import tracks_to_albums
+from lollypop.logger import Logger
 
 
 class FilteringHelper():
@@ -84,6 +86,64 @@ class FilteringHelper():
                 GLib.idle_add(self._scroll_to_child, child)
                 break
 
+    def activate_child(self):
+        """
+            Activated typeahead row
+        """
+        def reset_party_mode():
+            if App().player.is_party:
+                App().lookup_action("party").change_state(
+                    GLib.Variant("b", False))
+
+        try:
+            # Search typeahead child
+            typeahead_child = None
+            for child in self.filtered:
+                style_context = child.get_style_context()
+                if style_context.has_class("typeahead"):
+                    typeahead_child = child
+                    break
+            if typeahead_child is None:
+                return
+            from lollypop.view_current_albums import CurrentAlbumsView
+            from lollypop.view_playlists import PlaylistsView
+            # Play child without reseting player
+            if isinstance(self, CurrentAlbumsView):
+                if hasattr(typeahead_child, "album"):
+                    album = typeahead_child.album
+                    if album.tracks:
+                        track = album.tracks[0]
+                        App().player.load(track)
+                elif hasattr(typeahead_child, "track"):
+                    App().player.load(typeahead_child.track)
+            # Play playlist
+            elif isinstance(self, PlaylistsView):
+                reset_party_mode()
+                tracks = []
+                for album_row in self.view.children:
+                    for track in album_row.album.tracks:
+                        tracks.append(track)
+                if tracks:
+                    albums = tracks_to_albums(tracks)
+                    if hasattr(typeahead_child, "album"):
+                        if typeahead_child.album.tracks:
+                            App().player.play_track_for_albums(
+                                typeahead_child.album.tracks[0], albums)
+                    elif hasattr(typeahead_child, "track"):
+                        App().player.play_track_for_albums(
+                            typeahead_child.track, albums)
+            elif hasattr(typeahead_child, "data"):
+                reset_party_mode()
+                typeahead_child.activate()
+            elif hasattr(typeahead_child, "track"):
+                reset_party_mode()
+                track = typeahead_child.track
+                App().player.add_album(track.album)
+                App().player.load(track.album.get_track(track.id))
+            App().window.container.type_ahead.entry.set_text("")
+        except Exception as e:
+            Logger.error("View::activate_child: %s" % e)
+
     @property
     def filtered(self):
         """
@@ -95,20 +155,11 @@ class FilteringHelper():
     @property
     def scroll_shift(self):
         """
-            Add scroll shift on y axes
+            Get scroll shift for y axes
             @return int
         """
         return 0
 
-    @property
-    def scroll_relative_to(self):
-        """
-            Relative to scrolled widget
-            @return Gtk.Widget
-        """
-        # Case in most Lollypop views
-        return self._box
-
 #######################
 # PROTECTED           #
 #######################
@@ -117,14 +168,14 @@ class FilteringHelper():
             Scroll to child
             @param child as Gtk.Widget
         """
-        if child == self.__last_scrolled:
-            return
-        self.__last_scrolled = child
-        if self._view_type & ViewType.SCROLLED:
+        if self.view_type & ViewType.SCROLLED:
+            view_widget = self.scrolled.get_child()
+            if isinstance(view_widget, Gtk.Viewport):
+                view_widget = view_widget.get_child()
             coordinates = child.translate_coordinates(
-                self.scroll_relative_to, 0, -self.scroll_shift)
+                view_widget, 0, -self.scroll_shift)
             if coordinates:
-                self._scrolled.get_vadjustment().set_value(coordinates[1])
+                self.scrolled.get_vadjustment().set_value(coordinates[1])
 
 #######################
 # PRIVATE             #
