@@ -18,21 +18,31 @@ from urllib.parse import urlparse
 import unicodedata
 import cairo
 import time
+import re
+from hashlib import md5
 from threading import current_thread
 from functools import wraps
 
 from lollypop.logger import Logger
 from lollypop.define import App, Type, NetworkAccessACL
 from lollypop.define import StorageType
+from lollypop.shown import ShownLists
 
 
-def cancellable_sleep(seconds, cancellable):
-    if not App().debug:
-        while seconds > 0:
-            time.sleep(0.1)
-            seconds -= 0.1
-            if cancellable.is_cancelled():
-                raise Exception("cancelled")
+def make_subrequest(value, operand, count):
+    """
+        Make a subrequest for value and operand
+        @param value as str   => SQL
+        @param operand as str => OR/AND
+        @param count as int => iteration count
+    """
+    subrequest = "("
+    while count != 0:
+        if subrequest != "(":
+            subrequest += " %s " % operand
+        subrequest += value
+        count -= 1
+    return subrequest + ")"
 
 
 def ms_to_string(duration):
@@ -171,6 +181,10 @@ def init_proxy_from_gnome():
             socks = Gio.Settings.new("org.gnome.system.proxy.socks")
             host = socks.get_value("host").get_string()
             port = socks.get_value("port").get_int32()
+            proxy = "socks4://%s:%s" % (host, port)
+            from os import environ
+            environ["all_proxy"] = proxy
+            environ["ALL_PROXY"] = proxy
             if host != "" and port != 0:
                 import socket
                 import socks
@@ -261,6 +275,49 @@ def escape(str, ignore=["_", "-", " ", "."]):
                     c.isdigit() or c in ignore]).rstrip()
 
 
+def get_lollypop_album_id(name, artists):
+    """
+        Calculate Lollypop album id
+        @param name as str
+        @param artists as [str]
+    """
+    name = "%s_%s" % (sql_escape(" ".join(artists)), sql_escape(name))
+    return md5(name.encode("utf-8")).hexdigest()
+
+
+def get_lollypop_track_id(name, artists, album_name):
+    """
+        Calculate Lollypop track id
+        @param name as str
+        @param artists as [str]
+        @param year as str
+        @param album_name as str
+    """
+    name = "%s_%s_%s" % (sql_escape(" ".join(artists)), sql_escape(name),
+                         sql_escape(album_name))
+    return md5(name.encode("utf-8")).hexdigest()
+
+
+def get_iso_date_from_string(string):
+    """
+        Convert any string to an iso date
+        @param string as str
+        @return str/None
+    """
+    model = ["1970", "01", "01", "00", "00", "00"]
+    try:
+        split = re.split('[-:TZ]', string)
+        length = len(split)
+        while length < 6:
+            split.append(model[length])
+            length = len(split)
+        return "%s-%s-%sT%s:%s:%sZ" % (split[0], split[1], split[2],
+                                       split[3], split[4], split[5])
+    except Exception as e:
+        Logger.error("get_iso_date_from_string(): %s -> %s", string, e)
+        return None
+
+
 def is_unity():
     """
         Return True if desktop is Unity
@@ -275,71 +332,6 @@ def is_gnome():
     return GLib.getenv("XDG_CURRENT_DESKTOP") in ["ubuntu:GNOME", "GNOME"]
 
 
-def is_audio(f):
-    """
-        Return True if files is audio
-        @param f as Gio.File
-    """
-    audio = ["application/ogg", "application/x-ogg", "application/x-ogm-audio",
-             "audio/aac", "audio/mp4", "audio/mpeg", "audio/mpegurl",
-             "audio/ogg", "audio/vnd.rn-realaudio", "audio/vorbis",
-             "audio/x-flac", "audio/x-mp3", "audio/x-mpeg", "audio/x-mpegurl",
-             "audio/x-ms-wma", "audio/x-musepack", "audio/x-oggflac",
-             "audio/x-pn-realaudio", "application/x-flac", "audio/x-speex",
-             "audio/x-vorbis", "audio/x-vorbis+ogg", "audio/x-wav",
-             "x-content/audio-player", "audio/x-aac", "audio/m4a",
-             "audio/x-m4a", "audio/mp3", "audio/ac3", "audio/flac",
-             "audio/x-opus+ogg", "application/x-extension-mp4", "audio/x-ape",
-             "audio/x-pn-aiff", "audio/x-pn-au", "audio/x-pn-wav",
-             "audio/x-pn-windows-acm", "application/x-matroska",
-             "audio/x-matroska", "audio/x-wavpack", "video/mp4",
-             "audio/x-mod", "audio/x-mo3", "audio/x-xm", "audio/x-s3m",
-             "audio/x-it", "audio/aiff", "audio/x-aiff"]
-    try:
-        info = f.query_info("standard::content-type",
-                            Gio.FileQueryInfoFlags.NONE)
-        if info is not None:
-            content_type = info.get_content_type()
-            if content_type in audio:
-                return True
-    except Exception as e:
-        Logger.error("is_audio: %s", e)
-    return False
-
-
-def is_pls(f):
-    """
-        Return True if files is a playlist
-        @param f as Gio.File
-    """
-    try:
-        info = f.query_info("standard::content-type",
-                            Gio.FileQueryInfoFlags.NONE)
-        if info is not None:
-            if info.get_content_type() in ["audio/x-mpegurl",
-                                           "application/xspf+xml"]:
-                return True
-    except:
-        pass
-    return False
-
-
-def get_mtime(info):
-    """
-        Return Last modified time of a given file
-        @param info as Gio.FileInfo
-    """
-    # Using time::changed is not reliable making lollypop doing a full
-    # scan every two weeks (on my computer)
-    # try:
-    #    # We do not use time::modified because many tag editors
-    #    # just preserve this setting
-    #    return int(info.get_attribute_as_string("time::changed"))
-    # except:
-    #    pass
-    return int(info.get_attribute_as_string("time::modified"))
-
-
 def format_artist_name(name):
     """
         Return formated artist name
@@ -381,21 +373,6 @@ def translate_artist_name(name):
     return name
 
 
-def tracks_to_albums(tracks):
-    """
-        Convert tracks list to albums list
-    """
-    albums = []
-    for track in tracks:
-        if albums and albums[-1].id == track.album.id:
-            albums[-1].append_track(track, False)
-        else:
-            album = track.album
-            album.set_tracks([track], False)
-            albums.append(album)
-    return albums
-
-
 def get_page_score(page_title, title, artist, album):
     """
         Calculate web page score
@@ -422,33 +399,6 @@ def get_page_score(page_title, title, artist, album):
     return len(page_title)
 
 
-def is_readonly(uri):
-    """
-        Check if uri is readonly
-    """
-    try:
-        f = Gio.File.new_for_uri(uri)
-        info = f.query_info("access::can-write",
-                            Gio.FileQueryInfoFlags.NONE,
-                            None)
-        return not info.get_attribute_boolean("access::can-write")
-    except:
-        return True
-
-
-def create_dir(path):
-    """
-        Create dir
-        @param path as str
-    """
-    d = Gio.File.new_for_path(path)
-    if not d.query_exists():
-        try:
-            d.make_directory_with_parents()
-        except:
-            Logger.info("Can't create %s" % path)
-
-
 def remove_static(ids):
     """
         Remove static ids
@@ -517,27 +467,60 @@ def get_icon_name(object_id):
     return icon
 
 
-def popup_widget(widget, parent, x=None, y=None):
+def get_title_for_genres_artists(genre_ids, artist_ids):
+    """
+        Return title for genres/artists
+        @param genre_ids as [int]
+        @param artist_ids as [int]
+        @return str
+    """
+    if genre_ids and genre_ids[0] == Type.YEARS and artist_ids:
+        title_str = "%s - %s" % (artist_ids[0], artist_ids[-1])
+    else:
+        genres = []
+        for genre_id in genre_ids:
+            if genre_id < 0:
+                genres.append(ShownLists.IDS[genre_id])
+            else:
+                genre = App().genres.get_name(genre_id)
+                if genre is not None:
+                    genres.append(genre)
+        title_str = ",".join(genres)
+    return title_str
+
+
+def popup_widget(widget, parent, x, y, state_widget):
     """
         Popup menu on widget as x, y
         @param widget as Gtk.Widget
         @param parent as Gtk.Widget
         @param x as int
         @param y as int
+        @param state_widget as Gtk.Widget
         @return Gtk.Popover/None
     """
     def on_hidden(widget, hide, popover):
         popover.popdown()
 
+    def on_unmap(popover, parent):
+        parent.unset_state_flags(Gtk.StateFlags.VISITED)
+
     if App().window.is_adaptive:
         App().window.container.show_menu(widget)
         return None
     else:
-        from lollypop.widgets_utils import Popover
+        from lollypop.widgets_popover import Popover
         popover = Popover()
         popover.add(widget)
         widget.connect("hidden", on_hidden, popover)
+        if state_widget is not None:
+            popover.connect("unmap", on_unmap, state_widget)
+            state_widget.set_state_flags(Gtk.StateFlags.VISITED, False)
         popover.set_relative_to(parent)
+        # Workaround a GTK autoscrolling issue in Gtk.ListBox
+        # Gtk autoscroll to last focused widget on popover close
+        if state_widget is not None:
+            state_widget.grab_focus()
         if x is not None and y is not None:
             rect = Gdk.Rectangle()
             rect.x = x
@@ -580,7 +563,7 @@ def profile(f):
         ret = f(*args, **kwargs)
 
         elapsed_time = time.perf_counter() - start_time
-        Logger.debug("%s::%s: execution time %d:%f" % (
+        Logger.info("%s::%s: execution time %d:%f" % (
             f.__module__, f.__name__, elapsed_time / 60, elapsed_time % 60))
 
         return ret
@@ -588,77 +571,12 @@ def profile(f):
     return wrapper
 
 
-def install_youtube_dl():
-    try:
-        path = GLib.get_user_data_dir() + "/lollypop/python"
-        argv = ["pip3", "install", "-t", path, "-U", "youtube-dl"]
-        GLib.spawn_sync(None, argv, [], GLib.SpawnFlags.SEARCH_PATH, None)
-    except Exception as e:
-        Logger.error("install_youtube_dl: %s" % e)
-
-
-def get_youtube_dl():
+def split_list(l, n=1):
     """
-        Get youtube-dl path and env
-        @return (str, [])
+        Split list in n parts
+        @param l as []
+        @param n as int
     """
-    if App().settings.get_value("recent-youtube-dl"):
-        python_path = GLib.get_user_data_dir() + "/lollypop/python"
-        path = "%s/bin/youtube-dl" % python_path
-        env = ["PYTHONPATH=%s" % python_path]
-        f = Gio.File.new_for_path(path)
-        if f.query_exists():
-            return (path, env)
-    if GLib.find_program_in_path("youtube-dl"):
-        return ("youtube-dl", [])
-    else:
-        return (None, [])
-
-
-# From eyeD3 start
-# eyeD3 is written and maintained by:
-# Travis Shirk <travis@pobox.com>
-def id3EncodingToString(encoding):
-    from lollypop.define import LATIN1_ENCODING, UTF_8_ENCODING
-    from lollypop.define import UTF_16_ENCODING, UTF_16BE_ENCODING
-    if encoding == LATIN1_ENCODING:
-        return "latin_1"
-    elif encoding == UTF_8_ENCODING:
-        return "utf_8"
-    elif encoding == UTF_16_ENCODING:
-        return "utf_16"
-    elif encoding == UTF_16BE_ENCODING:
-        return "utf_16_be"
-    else:
-        raise ValueError("Encoding unknown: %s" % encoding)
-
-
-def decodeUnicode(bites, encoding):
-    codec = id3EncodingToString(encoding)
-    Logger.debug("Unicode encoding: %s" % codec)
-    if (codec.startswith("utf_16") and
-            len(bites) % 2 != 0 and bites[-1:] == b"\x00"):
-        # Catch and fix bad utf16 data, it is everywhere.
-        Logger.warning("Fixing utf16 data with extra zero bytes")
-        bites = bites[:-1]
-    return bites.decode(codec).rstrip("\x00")
-
-
-def splitUnicode(data, encoding):
-    from lollypop.define import LATIN1_ENCODING, UTF_8_ENCODING
-    from lollypop.define import UTF_16_ENCODING, UTF_16BE_ENCODING
-    try:
-        if encoding == LATIN1_ENCODING or encoding == UTF_8_ENCODING:
-            (d, t) = data.split(b"\x00", 1)
-        elif encoding == UTF_16_ENCODING or encoding == UTF_16BE_ENCODING:
-            # Two null bytes split, but since each utf16 char is also two
-            # bytes we need to ensure we found a proper boundary.
-            (d, t) = data.split(b"\x00\x00", 1)
-            if (len(d) % 2) != 0:
-                (d, t) = data.split(b"\x00\x00\x00", 1)
-                d += b"\x00"
-    except ValueError as ex:
-        Logger.warning("Invalid 2-tuple ID3 frame data: %s", ex)
-        d, t = data, b""
-    return (d, t)
-# From eyeD3 end
+    length = len(l)
+    split = [l[i * length // n: (i + 1) * length // n] for i in range(n)]
+    return [l for l in split if l]
