@@ -10,24 +10,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib
-
-from lollypop.utils import tracks_to_albums
-from lollypop.view_lazyloading import LazyLoadingView
+from lollypop.utils_album import tracks_to_albums
+from lollypop.utils import get_default_storage_type
 from lollypop.define import App, ViewType, MARGIN, Type, Size
 from lollypop.objects_album import Album
 from lollypop.objects_track import Track
-from lollypop.controller_view import ViewController, ViewControllerType
 from lollypop.widgets_banner_playlist import PlaylistBannerWidget
 from lollypop.view_albums_list import AlbumsListView
-from lollypop.logger import Logger
-from lollypop.helper_filtering import FilteringHelper
 from lollypop.helper_signals import SignalsHelper, signals_map
 from lollypop.helper_size_allocation import SizeAllocationHelper
 
 
-class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
-                    SignalsHelper, SizeAllocationHelper):
+class PlaylistsView(AlbumsListView, SignalsHelper, SizeAllocationHelper):
     """
         View showing playlists
     """
@@ -39,92 +33,36 @@ class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
             @parma playlist_id as int
             @param view_type as ViewType
         """
-        LazyLoadingView.__init__(self, view_type |
-                                 ViewType.SCROLLED |
-                                 ViewType.OVERLAY)
-        ViewController.__init__(self, ViewControllerType.ALBUM)
-        FilteringHelper.__init__(self)
+        AlbumsListView.__init__(self, [], [], view_type |
+                                ViewType.SCROLLED |
+                                ViewType.OVERLAY)
         SizeAllocationHelper.__init__(self)
-        self._playlist_id = playlist_id
-        # We remove SCROLLED because we want to be the scrolled view
-        self._view = AlbumsListView([], [], view_type & ~ViewType.SCROLLED)
-        self._view.set_width(Size.MEDIUM)
+        self.__playlist_id = playlist_id
+        self.box.set_width(Size.MEDIUM)
         if view_type & ViewType.DND:
-            self._view.dnd_helper.connect("dnd-finished",
-                                          self.__on_dnd_finished)
-        self._view.show()
-        self._banner = PlaylistBannerWidget(playlist_id, self._view)
-        self._banner.show()
-        self.add_widget(self._view, self._banner)
+            self.dnd_helper.connect("dnd-finished",
+                                    self.__on_dnd_finished)
+        self.__banner = PlaylistBannerWidget(playlist_id, self)
+        self.__banner.show()
+        self.add_widget(self.box, self.__banner)
         return [
                 (App().playlists, "playlist-track-added",
                  "_on_playlist_track_added"),
                 (App().playlists, "playlist-track-removed",
                  "_on_playlist_track_removed"),
-                (App().playlists, "playlists-changed", "_on_playlist_changed")
+                (App().playlists, "playlists-removed", "_on_playlist_removed"),
+                (App().playlists, "playlists-renamed", "_on_playlist_renamed"),
+                (App().playlists, "playlists-updated", "_on_playlist_updated")
         ]
 
     def populate(self):
         """
             Populate view
         """
-        def on_load(albums):
-            self._view.add_reveal_albums(albums)
-            self._view.populate(albums)
-
-        def load():
-            track_ids = []
-            if self._playlist_id == Type.LOVED:
-                for track_id in App().tracks.get_loved_track_ids():
-                    if track_id not in track_ids:
-                        track_ids.append(track_id)
-            else:
-                for track_id in App().playlists.get_track_ids(
-                        self._playlist_id):
-                    if track_id not in track_ids:
-                        track_ids.append(track_id)
-            return tracks_to_albums(
-                [Track(track_id) for track_id in track_ids])
-
-        App().task_helper.run(load, callback=(on_load,))
-
-    def pause(self):
-        """
-            pause populating
-        """
-        self._view.pause()
-
-    def activate_child(self):
-        """
-            Activated typeahead row
-        """
-        try:
-            if App().player.is_party:
-                App().lookup_action("party").change_state(
-                    GLib.Variant("b", False))
-            for child in self.filtered:
-                style_context = child.get_style_context()
-                if style_context.has_class("typeahead"):
-                    if hasattr(child, "album"):
-                        App().player.play_album(child.album)
-                    else:
-                        track = child.track
-                        App().player.add_album(track.album)
-                        App().player.load(track.album.get_track(track.id))
-                style_context.remove_class("typeahead")
-        except Exception as e:
-            Logger.error("PlaylistsView::activate_child: %s" % e)
-
-    def remove_from_playlist(self, object):
-        """
-            Remove object from playlist
-            @param object as Album/Track
-        """
-        if isinstance(object, Album):
-            tracks = object.tracks
+        if App().playlists.get_smart(self.__playlist_id):
+            self.__populate_smart()
         else:
-            tracks = [object]
-        App().playlists.remove_tracks(self._playlist_id, tracks)
+            self.__populate()
 
     @property
     def args(self):
@@ -132,7 +70,8 @@ class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
             Get default args for __class__
             @return {}
         """
-        return {"playlist_id": self._playlist_id, "view_type": self.view_type}
+        return {"playlist_id": self.__playlist_id,
+                "view_type": self.view_type}
 
     @property
     def filtered(self):
@@ -141,7 +80,7 @@ class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
             @return [Gtk.Widget]
         """
         filtered = []
-        for child in self._view.children:
+        for child in self.children:
             filtered.append(child)
             for subchild in child.children:
                 filtered.append(subchild)
@@ -150,18 +89,10 @@ class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
     @property
     def scroll_shift(self):
         """
-            Add scroll shift on y axes
+            Get scroll shift for y axes
             @return int
         """
-        return self._banner.height + MARGIN
-
-    @property
-    def scroll_relative_to(self):
-        """
-            Relative to scrolled widget
-            @return Gtk.Widget
-        """
-        return self._view
+        return self.__banner.height + MARGIN
 
 #######################
 # PROTECTED           #
@@ -173,12 +104,12 @@ class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
             @param playlist_id as int
             @param uri as str
         """
-        if playlist_id == self._playlist_id:
+        if playlist_id == self.__playlist_id:
             track = Track(App().tracks.get_id_by_uri(uri))
             album = Album(track.album.id)
             album.set_tracks([track])
-            self._view.add_reveal_albums([album])
-            self._view.insert_album(album, -1)
+            self.add_reveal_albums([album])
+            self.add_value(album)
 
     def _on_playlist_track_removed(self, playlists, playlist_id, uri):
         """
@@ -187,76 +118,138 @@ class PlaylistsView(FilteringHelper, LazyLoadingView, ViewController,
             @param playlist_id as int
             @param uri as str
         """
-        if playlist_id == self._playlist_id:
+        if playlist_id == self.__playlist_id:
             track = Track(App().tracks.get_id_by_uri(uri))
-            children = self._view.children
-            for album_row in children:
+            for album_row in self.children:
                 if album_row.album.id == track.album.id:
                     for track_row in album_row.children:
                         if track_row.track.id == track.id:
                             track_row.destroy()
-                            if len(children) == 1:
+                            if len(self.children) == 1:
                                 album_row.destroy()
                                 break
 
-    def _on_playlist_changed(self, playlists, playlist_id):
+    def _on_playlist_updated(self, playlists, playlist_id):
         """
-            Update playlist
+            Reload view
             @param playlists as Playlists
             @param playlist_id as int
         """
-        if playlist_id == self._playlist_id:
-            # Playlist has been removed
-            if not playlists.exists(playlist_id):
-                App().window.container.go_back()
-            else:
-                self._view.clear()
-                self.populate()
+        self.clear()
+        self.populate()
+
+    def _on_playlist_removed(self, playlists, playlist_id):
+        """
+            Go back
+            @param playlists as Playlists
+            @param playlist_id as int
+        """
+        App().window.container.go_back()
+
+    def _on_playlist_renamed(self, playlists, playlist_id):
+        """
+            Update banner
+            @param playlists as Playlists
+            @param playlist_id as int
+        """
+        self.__banner.rename(App().playlists.get_name(playlist_id))
+
+    def _on_row_activated(self, row, track):
+        """
+            Start playback
+            @param row as AlbumRow
+            @param track_id as int
+        """
+        if App().player.is_party:
+            App().player.load(track)
+        else:
+            albums = []
+            for album_row in self.children:
+                albums.append(album_row.album)
+            App().player.play_track_for_albums(track, albums)
+
+    def _on_row_destroy(self, row):
+        """
+            Remove album from playback
+            @param row as AlbumRow
+        """
+        App().playlists.remove_tracks(self.__playlist_id,
+                                      row.album.tracks,
+                                      True)
+
+    def _on_track_removed(self, row, track):
+        """
+            @param row as AlbumRow
+            @param track as Track
+        """
+        row.album.remove_track(track)
+        App().playlists.remove_tracks(self.__playlist_id,
+                                      [track],
+                                      True)
+        if not row.children:
+            row.destroy()
 
 #######################
 # PRIVATE             #
 #######################
+    def __populate(self):
+        """
+            Populate view
+        """
+        def on_load(albums):
+            self.add_reveal_albums(albums)
+            AlbumsListView.populate(self, albums)
+
+        def load():
+            track_ids = []
+            if self.__playlist_id == Type.LOVED:
+                for track_id in App().tracks.get_loved_track_ids(
+                        [],
+                        self.storage_type):
+                    if track_id not in track_ids:
+                        track_ids.append(track_id)
+            else:
+                for track_id in App().playlists.get_track_ids(
+                        self.__playlist_id):
+                    if track_id not in track_ids:
+                        track_ids.append(track_id)
+            return tracks_to_albums(
+                [Track(track_id) for track_id in track_ids])
+
+        App().task_helper.run(load, callback=(on_load,))
+
+    def __populate_smart(self):
+        """
+            Populate view
+        """
+        def on_load(albums):
+            self.banner.spinner.stop()
+            self.add_reveal_albums(albums)
+            AlbumsListView.populate(self, albums)
+
+        def load():
+            request = App().playlists.get_smart_sql(self.__playlist_id)
+            # We need to inject skipped/storage_type
+            storage_type = get_default_storage_type()
+            split = request.split("ORDER BY")
+            split[0] += " AND tracks.loved != %s" % Type.NONE
+            split[0] += " AND tracks.storage_type&%s " % storage_type
+            track_ids = App().db.execute("ORDER BY".join(split))
+            return tracks_to_albums(
+                [Track(track_id) for track_id in track_ids])
+
+        self.banner.spinner.start()
+        App().task_helper.run(load, callback=(on_load,))
+
     def __on_dnd_finished(self, dnd_helper):
         """
             Save playlist if needed
             @param dnd_helper as DNDHelper
         """
-        if self._playlist_id >= 0:
+        if self.__playlist_id >= 0:
             uris = []
-            for child in self._view.children:
+            for child in self.children:
                 for track in child.album.tracks:
                     uris.append(track.uri)
-            App().playlists.clear(self._playlist_id)
-            App().playlists.add_uris(self._playlist_id, uris)
-
-
-class SmartPlaylistsView(PlaylistsView):
-    """
-        View showing smart playlists
-    """
-
-    def __init__(self, playlist_id, view_type):
-        """
-            Init PlaylistView
-            @parma playlist_id as int
-            @param view_type as ViewType
-        """
-        PlaylistsView.__init__(self, playlist_id, view_type)
-
-    def populate(self):
-        """
-            Populate view
-        """
-        def on_load(albums):
-            self._banner.spinner.stop()
-            self._view.add_reveal_albums(albums)
-            self._view.populate(albums)
-
-        def load():
-            request = App().playlists.get_smart_sql(self._playlist_id)
-            track_ids = App().db.execute(request)
-            return tracks_to_albums(
-                [Track(track_id) for track_id in track_ids])
-
-        self._banner.spinner.start()
-        App().task_helper.run(load, callback=(on_load,))
+            App().playlists.clear(self.__playlist_id)
+            App().playlists.add_uris(self.__playlist_id, uris)
