@@ -17,25 +17,26 @@ from gettext import gettext as _
 from lollypop.view import View
 from lollypop.utils import get_network_available
 from lollypop.define import ViewType, StorageType, Size, App
-from lollypop.helper_filtering import FilteringHelper
 from lollypop.view_albums_line import AlbumsPopularsLineView
 from lollypop.view_albums_line import AlbumsRandomGenresLineView
 from lollypop.view_artists_line import ArtistsRandomLineView
 from lollypop.widgets_banner_today import TodayBannerWidget
+from lollypop.helper_signals import signals_map
 
 
-class SuggestionsView(FilteringHelper, View):
+class SuggestionsView(View):
     """
         View showing suggestions to user
     """
 
-    def __init__(self, view_type=ViewType.DEFAULT):
+    @signals_map
+    def __init__(self, storage_type, view_type):
         """
             Init view
+            @param storage_type as StorageType
             @param view_type as ViewType
         """
-        View.__init__(self, view_type | ViewType.SCROLLED | ViewType.OVERLAY)
-        FilteringHelper.__init__(self)
+        View.__init__(self, storage_type, view_type | ViewType.OVERLAY)
         self.__grid = Gtk.Grid()
         self.__grid.get_style_context().add_class("padding")
         self.__grid.set_row_spacing(5)
@@ -47,45 +48,26 @@ class SuggestionsView(FilteringHelper, View):
         else:
             album = TodayBannerWidget.get_today_album()
             if album is not None:
-                self.__banner = TodayBannerWidget(album, self._view_type)
+                self.__banner = TodayBannerWidget(album, self.view_type)
                 self.__banner.show()
             else:
                 self.__banner = None
             self.add_widget(self.__grid, self.__banner)
+        return [
+            (App().settings, "changed::suggestions-mask", "_on_mask_changed"),
+        ]
 
     def populate(self):
         """
             Populate view
         """
-        for _class in [AlbumsPopularsLineView,
-                       ArtistsRandomLineView,
-                       AlbumsRandomGenresLineView]:
-            view = _class(self._view_type)
+        for cls in [AlbumsPopularsLineView,
+                    ArtistsRandomLineView,
+                    AlbumsRandomGenresLineView]:
+            view = cls(self.storage_type, self.view_type)
             view.populate()
             self.__grid.add(view)
-        if get_network_available("SPOTIFY") and\
-                get_network_available("YOUTUBE"):
-            from lollypop.view_albums_line import AlbumsSpotifyLineView
-            spotify_view = AlbumsSpotifyLineView(_("You might like this"),
-                                                 self._view_type)
-            spotify_view.populate(StorageType.SPOTIFY_SIMILARS)
-            self.__grid.add(spotify_view)
-            spotify_view = AlbumsSpotifyLineView(_("New albums from Spotify"),
-                                                 self._view_type)
-            spotify_view.populate(StorageType.SPOTIFY_NEW_RELEASES)
-            self.__grid.add(spotify_view)
-
-    def activate_child(self):
-        """
-            Activated typeahead row
-        """
-        for child in reversed(self.__grid.get_children()):
-            child._box.unselect_all()
-        for row in self.filtered:
-            style_context = row.get_style_context()
-            if style_context.has_class("typeahead"):
-                row.activate()
-            style_context.remove_class("typeahead")
+        self.__add_web_views()
 
     @property
     def filtered(self):
@@ -98,25 +80,64 @@ class SuggestionsView(FilteringHelper, View):
             children += child.children
         return children
 
-    @property
-    def scroll_relative_to(self):
-        """
-            Relative to scrolled widget
-            @return Gtk.Widget
-        """
-        return self
-
     @property
     def args(self):
         """
             Get default args for __class__
             @return {}
         """
-        return {"view_type": self.view_type}
+        return {"storage_type": self.storage_type,
+                "view_type": self.view_type}
+
+#######################
+# PROTECTED           #
+#######################
+    def _on_mask_changed(self, settings, value):
+        """
+            Reload web views
+            @param settings as Gio.Settings
+            @param value as str
+        """
+        self.__add_web_views()
+
+    def _on_map(self, widget):
+        """
+            Set initial view state
+            @param widget as GtK.Widget
+        """
+        View._on_map(self, widget)
+        if self.view_type & ViewType.SCROLLED:
+            self.scrolled.grab_focus()
 
 #######################
 # PRIVATE             #
 #######################
+    def __add_web_views(self):
+        """
+            Add web views
+        """
+        from lollypop.view_albums_line import AlbumsStorageTypeLineView
+        for child in self.__grid.get_children():
+            if isinstance(child, AlbumsStorageTypeLineView):
+                child.destroy()
+
+        mask = App().settings.get_value("suggestions-mask").get_int32()
+        if get_network_available("YOUTUBE"):
+            for (title, storage_type) in [
+                   (_("Suggestions from Spotify"),
+                    StorageType.SPOTIFY_SIMILARS),
+                   (_("New releases on Spotify"),
+                    StorageType.SPOTIFY_NEW_RELEASES),
+                   (_("Top albums on Deezer"),
+                    StorageType.DEEZER_CHARTS)]:
+                if not storage_type & mask:
+                    continue
+                view = AlbumsStorageTypeLineView(title,
+                                                 self.storage_type,
+                                                 self.view_type)
+                view.populate(storage_type)
+                self.__grid.add(view)
+
     def __welcome_screen(self):
         """
             Show welcome screen if view empty
