@@ -15,8 +15,7 @@ from gi.repository import Gio, GObject, Gtk
 from lollypop.logger import Logger
 from lollypop.utils import emit_signal
 from lollypop.widgets_artwork import ArtworkSearchWidget, ArtworkSearchChild
-from lollypop.define import App, ViewType, ArtSize, ArtBehaviour, MARGIN
-from lollypop.define import MARGIN_SMALL
+from lollypop.define import App, ArtSize, StorageType
 
 
 class ArtistArtworkSearchWidget(ArtworkSearchWidget):
@@ -28,7 +27,7 @@ class ArtistArtworkSearchWidget(ArtworkSearchWidget):
         "hidden": (GObject.SignalFlags.RUN_FIRST, None, (bool,)),
     }
 
-    def __init__(self, artist_id, view_type):
+    def __init__(self, artist_id, view_type, in_menu=False):
         """
             Init search
             @param artist_id as int
@@ -36,40 +35,6 @@ class ArtistArtworkSearchWidget(ArtworkSearchWidget):
         """
         ArtworkSearchWidget.__init__(self, view_type)
         self.__artist = App().artists.get_name(artist_id)
-        if view_type & ViewType.ADAPTIVE:
-            self.set_row_spacing(MARGIN)
-            self.set_margin_start(MARGIN_SMALL)
-            self.set_margin_end(MARGIN_SMALL)
-            self.set_margin_top(MARGIN)
-            self.set_margin_bottom(MARGIN)
-            button = Gtk.ModelButton.new()
-            button.set_alignment(0, 0.5)
-            button.connect("clicked",
-                           lambda x: emit_signal(self, "hidden", True))
-            button.show()
-            label = Gtk.Label.new()
-            label.show()
-            self.__artwork = Gtk.Image.new()
-            name = "<span alpha='40000'>%s</span>" % self.__artist
-            App().art_helper.set_artist_artwork(
-                                       self.__artist,
-                                       ArtSize.SMALL,
-                                       ArtSize.SMALL,
-                                       self.__artwork.get_scale_factor(),
-                                       ArtBehaviour.ROUNDED |
-                                       ArtBehaviour.CROP_SQUARE |
-                                       ArtBehaviour.CACHE,
-                                       self.__on_artist_artwork)
-            self.__artwork.show()
-            label.set_markup(name)
-            grid = Gtk.Grid()
-            grid.set_column_spacing(MARGIN)
-            grid.add(self.__artwork)
-            grid.add(label)
-            button.set_image(grid)
-            button.get_style_context().add_class("padding")
-            self.insert_row(0)
-            self.attach(button, 0, 0, 1, 1)
 
 #######################
 # PROTECTED           #
@@ -83,7 +48,8 @@ class ArtistArtworkSearchWidget(ArtworkSearchWidget):
             f = Gio.File.new_for_path(filename)
             (status, data, tag) = f.load_contents()
             if status:
-                App().art.add_artist_artwork(self.__artist, data)
+                App().art.add_artist_artwork(self.__artist, data,
+                                             StorageType.COLLECTION)
         except Exception as e:
             Logger.error(
                 "ArtistArtworkSearchWidget::_save_from_filename(): %s" % e)
@@ -118,10 +84,12 @@ class ArtistArtworkSearchWidget(ArtworkSearchWidget):
         try:
             if isinstance(child, ArtworkSearchChild):
                 App().task_helper.run(App().art.add_artist_artwork,
-                                      self.__artist, child.bytes)
+                                      self.__artist, child.bytes,
+                                      StorageType.COLLECTION)
             else:
                 App().task_helper.run(App().art.add_artist_artwork,
-                                      self.__artist, None)
+                                      self.__artist, None,
+                                      StorageType.COLLECTION)
             emit_signal(self, "hidden", True)
         except Exception as e:
             Logger.error("ArtistArtworkSearchWidget::_on_activate(): %s", e)
