@@ -58,7 +58,7 @@ class BannerWidget(Gtk.Revealer, SizeAllocationHelper):
         """
         Gtk.Revealer.__init__(self)
         self._artwork = None
-        self._view_type = view_type
+        self.__view_type = view_type | ViewType.BANNER
         self.__width = 1
         self.set_property("valign", Gtk.Align.START)
         if view_type & ViewType.OVERLAY:
@@ -95,6 +95,13 @@ class BannerWidget(Gtk.Revealer, SizeAllocationHelper):
         if self._artwork is not None:
             self._artwork.set_size_request(-1, self.height)
 
+    def set_view_type(self, view_type):
+        """
+            Set view type
+            @param view_type as ViewType
+        """
+        self.__view_type = view_type | ViewType.BANNER
+
     @property
     def width(self):
         """
@@ -117,14 +124,20 @@ class BannerWidget(Gtk.Revealer, SizeAllocationHelper):
             Get wanted height
             @return int
         """
-        if self.width <= Size.SMALL:
-            height = ArtSize.MEDIUM
-        elif self.width <= Size.MEDIUM:
+        if self.width <= Size.MEDIUM:
             height = ArtSize.MEDIUM + MARGIN * 2
         else:
             height = ArtSize.BANNER + MARGIN * 2
         return height
 
+    @property
+    def view_type(self):
+        """
+            Get view type
+            @return ViewType
+        """
+        return self.__view_type
+
 #######################
 # PROTECTED           #
 #######################
